@@ -2,6 +2,7 @@ import time
 import logging
 import functools
 import json
+import uuid
 
 import bizerror
 from fastutils import rsautils
@@ -17,6 +18,7 @@ from .utils import SimpleJsonEncoder
 from .utils import get_request_data
 from .pack import SimpleJsonResultPacker
 from .pack import SafeJsonResultPacker
+from .models import ApiResponseTimeLog
 
 logger = logging.getLogger(__name__)
 
@@ -61,10 +63,13 @@ class ApiviewRequest(object):
             return funcutils.call_with_inject(func, self.data)
 
 class Apiview(object):
-    def __init__(self, packer, preload_plugins=None, extra_parameters=None):
+    def __init__(self, packer, preload_plugins=None, extra_parameters=None, enable_api_response_time_log=None):
         self.packer = packer
         self.extra_parameters = extra_parameters or {}
         self.preload_plugins = preload_plugins or []
+        if enable_api_response_time_log is None:
+            enable_api_response_time_log = getattr(settings, "DJANGO_APIVIEW_ENABLE_API_RESPONSE_TIME_LOG", False)
+        self.enable_api_response_time_log = enable_api_response_time_log
     
     def set_packer(self, packer):
         self.packer = packer
@@ -78,10 +83,21 @@ class Apiview(object):
     def put_extra_parameters(self, params):
         self.extra_parameters.update(params)
 
+    def log_api_response_time(self, path, response_time):
+        if self.enable_api_response_time_log:
+            try:
+                log = ApiResponseTimeLog()
+                log.path = path
+                log.response_time = int(response_time)
+                log.save()
+            except Exception:
+                logger.exception("Save api response time log failed.")
+
     def __call__(self, func):
         """Turn the view function into apiview function. Must use as the first decorator.
         """
         def wrapper(request, **kwargs):
+            stime = time.time()
             target_func = func
             defaults = {}
             defaults.update(self.extra_parameters)
@@ -98,9 +114,12 @@ class Apiview(object):
                 if not isinstance(error, bizerror.BizErrorBase):
                     error = bizerror.BizError(error)
                 package = self.packer.pack_error(error, **_django_apiview_request.data)
+            etime = time.time()
+            self.log_api_response_time(_django_apiview_request.request.path, 1000000*(etime-stime))
             return JsonResponse(package, encoder=SimpleJsonEncoder, json_dumps_params={"ensure_ascii": False, "allow_nan": True, "sort_keys": True})
         wrapper.csrf_exempt = True
         return functools.wraps(func)(wrapper)
+        
 
 apiview = Apiview(simple_result_packer)
 
@@ -247,7 +266,7 @@ class cache(ApiviewDecorator):
     def __init__(self, key, expire=None, cache_name="default", get_from_cache=None, set_to_cache=True, disable_get_from_cache_header=None, batch_mode=False, ignore_cache_errors=True, related_models=None):
         self.key = key
         self.expire = expire or getattr(settings, "DJANGO_APIVIEW_DEFAULT_CACHE_EXPIRE", None)
-        self.expire = self.expire or int(self.expire) or None
+        self.expire = self.expire and int(self.expire) or None
         self.cache_name = cache_name
         self.get_from_cache = get_from_cache
         self.set_to_cache = set_to_cache
@@ -376,20 +395,53 @@ def safe_apiview(packer_class=SafeJsonResultPacker, **kwargs):
         )
 
 
+DEFAULT_RANDOM_DJANGO_APIVIEW_ACLKEY = str(uuid.uuid4())
+logger.info("Default random DJANGO_APIVIEW_ACLKEY={0}".format(DEFAULT_RANDOM_DJANGO_APIVIEW_ACLKEY))
+
 class check_aclkey(ApiviewDecorator):
 
     def __init__(self, aclkey=None, aclkey_field_name="aclkey"):
         self.aclkey_field_name = aclkey_field_name
-        self.aclkey = aclkey or settings.DJANGO_APIVIEW_ACLKEY
+        self.aclkey = aclkey or getattr(settings, "DJANGO_APIVIEW_ACLKEY", DEFAULT_RANDOM_DJANGO_APIVIEW_ACLKEY)
     
     def main(self, _django_apiview_request, func):
+        if not self.aclkey_field_name in _django_apiview_request.data:
+            raise bizerror.MissingParameter(parameter=self.aclkey_field_name)
         aclkey = _django_apiview_request.data[self.aclkey_field_name]
-        if not self._check_aclkey(aclkey):
+        if not self._check_aclkey(aclkey, _django_apiview_request):
             raise bizerror.AppAuthFailed()
         return super().main(_django_apiview_request, func)
 
-    def _check_aclkey(self, aclkey):
+    def _check_aclkey(self, aclkey, _django_apiview_request):
         if callable(self.aclkey):
-            return self.aclkey() == aclkey
-        else:
-            return self.aclkey == aclkey
+            aclkey = funcutils.call_with_inject(self.aclkey, _django_apiview_request.data)
+        return self.aclkey == aclkey
+
+
+class string_length_limit(ApiviewDecorator):
+    """Check string parameter length.
+    """
+    def __init__(self, field, max_length, min_length=0, string_too_short_error_message=None, string_too_long_error_message=None):
+        self.field = field
+        self.max_length = max_length
+        self.min_length = min_length
+        self.string_too_short_error_message = string_too_short_error_message
+        self.string_too_long_error_message = string_too_long_error_message
+    
+    def main(self, _django_apiview_request, func):
+        value = _django_apiview_request.data.get(self.field, None)
+        if not value is None:
+            value_length = len(value)
+            if value_length < self.min_length:
+                raise bizerror.StringTooShort(message=self.string_too_short_error_message, min_length=self.min_length)
+            if value_length > self.max_length:
+                raise bizerror.StringTooLong(message=self.string_too_long_error_message, max_length=self.max_length)
+        return super().main(_django_apiview_request, func)
+
+
+
+@apiview
+@check_aclkey()
+def updateApiResponseTimeStats():
+    from .models import ApiResponseTimeStats
+    return ApiResponseTimeStats.update()
